@@ -485,7 +485,7 @@ int evutil_parse_sockaddr_port(const char *str, struct sockaddr *out, int *outle
 
 /**
  * Format an address to ipv4:port or [ipv6]:port
- * 
+ *
  * @param sa A struct sockaddr
  * @param out The output buffer
  * @param outlen The output buffer length
@@ -493,7 +493,7 @@ int evutil_parse_sockaddr_port(const char *str, struct sockaddr *out, int *outle
  */
 const char *evutil_format_sockaddr_port(const struct sockaddr *sa, char *out,
                                         size_t outlen);
-    
+
 /** Compare two sockaddrs; return 0 if they are equal, or less than 0 if sa1
  * preceeds sa2, or greater than 0 if sa1 follows sa2.  If include_port is
  * true, consider the port as well as the address.  Only implemented for
@@ -686,6 +686,21 @@ void evutil_secure_rng_get_bytes(void *buf, size_t n);
  */
 int evutil_secure_rng_init(void);
 
+/**
+ * Set a filename to use in place of /dev/urandom for seeding the secure
+ * PRNG. Return 0 on success, -1 on failure.
+ *
+ * Call this function BEFORE calling any other initialization or RNG
+ * functions.
+ *
+ * (This string will _NOT_ be copied internally. Do not free it while any
+ * user of the secure RNG might be running. Don't pass anything other than a
+ * real /dev/...random device file here, or you might lose security.)
+ *
+ * This API is unstable, and might change in a future libevent version.
+ */
+int evutil_secure_rng_set_urandom_device_file(char *fname);
+
 /** Seed the random number generator with extra random bytes.
 
     You should almost never need to call this function; it should be
