@@ -565,6 +565,11 @@ tcp_listener_bind(ProxyContext * const proxy_context)
     } else {
         evutil_make_socket_closeonexec(proxy_context->tcp_listener_handle);
         evutil_make_socket_nonblocking(proxy_context->tcp_listener_handle);
+#ifdef TCP_FASTOPEN
+        setsockopt(proxy_context->tcp_listener_handle,
+                   IPPROTO_TCP, TCP_FASTOPEN,
+                   (void *) (int[]) { TCP_FASTOPEN_QUEUES }, sizeof (int));
+#endif
         proxy_context->tcp_conn_listener =
             evconnlistener_new(proxy_context->event_loop,
                                tcp_connection_cb, proxy_context,
